@@ -369,6 +369,16 @@ CK_RV XProcUnLock(void)
 	return CKR_OK;
 }
 
+CK_RV XProcClose(void)
+{
+	if (xplfd != -1)
+		close(xplfd);
+	else
+		TRACE_DEVEL("XProcClose: No file descriptor open to close.\n");
+
+	return CKR_OK;
+}
+
 unsigned long AddToSessionList(ST_SESSION_T * pSess)
 {
 	unsigned long handle;
@@ -810,7 +820,7 @@ DLL_Load_t *dllload;
 
 	} else {
 		char *e = dlerror();
-		OCK_SYSLOG(LOG_ERR,
+		OCK_SYSLOG(LOG_WARNING,
 			   "%s: dlopen() failed for [%s]; dlerror = [%s]\n",
 			   __FUNCTION__, sinfp->dll_location, e);
 		TRACE_DEVEL("DL_Load of %s failed, dlerror: %s\n",
